searchData={"items":[{"type":"module","doc":"Building blocks for working with HTML in Phoenix.\n\nThis library provides three main functionalities:\n\n  * HTML safety\n  * Form abstractions\n  * A tiny JavaScript library to enhance applications","title":"Phoenix.HTML","ref":"Phoenix.HTML.html"},{"type":"module","doc":"One of the main responsibilities of this package is to\nprovide convenience functions for escaping and marking\nHTML code as safe.\n\nBy default, data output in templates is not considered\nsafe:\n\n```heex\n<%= \" \" %>\n```\n\nwill be shown as:\n\n```html\n&lt;hello&gt;\n```\n\nUser data or data coming from the database is almost never\nconsidered safe. However, in some cases, you may want to tag\nit as safe and show its \"raw\" contents:\n\n```heex\n<%= raw \" \" %>\n```","title":"HTML safety - Phoenix.HTML","ref":"Phoenix.HTML.html#module-html-safety"},{"type":"module","doc":"See `Phoenix.HTML.Form`.","title":"Form handling - Phoenix.HTML","ref":"Phoenix.HTML.html#module-form-handling"},{"type":"module","doc":"This project ships with a tiny bit of JavaScript that listens\nto all click events to:\n\n  * Support `data-confirm=\"message\"` attributes, which shows\n    a confirmation modal with the given message\n\n  * Support `data-method=\"patch|post|put|delete\"` attributes,\n    which sends the current click as a PATCH/POST/PUT/DELETE\n    HTTP request. You will need to add `data-to` with the URL\n    and `data-csrf` with the CSRF token value\n\n  * Dispatch a \"phoenix.link.click\" event. You can listen to this\n    event to customize the behaviour above. Returning false from\n    this event will disable `data-method`. Stopping propagation\n    will disable `data-confirm`\n\nTo use the functionality above, you must load `priv/static/phoenix_html.js`\ninto your build tool.","title":"JavaScript library - Phoenix.HTML","ref":"Phoenix.HTML.html#module-javascript-library"},{"type":"module","doc":"You can override the default implementation by hooking\ninto `phoenix.link.click`. Here is an example:\n\n```javascript\nwindow.addEventListener('phoenix.link.click', function (e) {\n  // Introduce custom behaviour\n  var message = e.target.getAttribute(\"data-prompt\");\n  var answer = e.target.getAttribute(\"data-prompt-answer\");\n  if(message && answer && (answer != window.prompt(message))) {\n    e.preventDefault();\n  }\n}, false);\n```","title":"Overriding the default confirmation behaviour - Phoenix.HTML","ref":"Phoenix.HTML.html#module-overriding-the-default-confirmation-behaviour"},{"type":"function","doc":"Escapes an enumerable of attributes, returning iodata.\n\nThe attributes are rendered in the given order. Note if\na map is given, the key ordering is not guaranteed.\n\nThe keys and values can be of any shape, as long as they\nimplement the `Phoenix.HTML.Safe` protocol. In addition,\nif the key is an atom, it will be \"dasherized\". In other\nwords, `:phx_value_id` will be converted to `phx-value-id`.\n\nFurthermore, the following attributes provide behaviour:\n\n  * `:aria`, `:data`, and `:phx` - they accept a keyword list as\n    value. `data: [confirm: \"are you sure?\"]` is converted to\n    `data-confirm=\"are you sure?\"`.\n\n  * `:class` - it accepts a list of classes as argument. Each\n    element in the list is separated by space. `nil` and `false`\n    elements are discarded. `class: [\"foo\", nil, \"bar\"]` then\n    becomes `class=\"foo bar\"`.\n\n  * `:id` - it is validated raise if a number is given as ID,\n    which is not allowed by the HTML spec and leads to unpredictable\n    behaviour.","title":"Phoenix.HTML.attributes_escape/1","ref":"Phoenix.HTML.html#attributes_escape/1"},{"type":"function","doc":"iex> safe_to_string attributes_escape(title: \"the title\", id: \"the id\", selected: true)\n    \" title=\\\"the title\\\" id=\\\"the id\\\" selected\"\n\n    iex> safe_to_string attributes_escape(%{data: [confirm: \"Are you sure?\"]})\n    \" data-confirm=\\\"Are you sure?\\\"\"\n\n    iex> safe_to_string attributes_escape(%{phx: [value: [foo: \"bar\"]]})\n    \" phx-value-foo=\\\"bar\\\"\"","title":"Examples - Phoenix.HTML.attributes_escape/1","ref":"Phoenix.HTML.html#attributes_escape/1-examples"},{"type":"function","doc":"Escapes a string for use as a CSS identifier.","title":"Phoenix.HTML.css_escape/1","ref":"Phoenix.HTML.html#css_escape/1"},{"type":"function","doc":"iex> css_escape(\"hello world\")\n    \"hello\\\\ world\"\n\n    iex> css_escape(\"-123\")\n    \"-\\\\31 23\"","title":"Examples - Phoenix.HTML.css_escape/1","ref":"Phoenix.HTML.html#css_escape/1-examples"},{"type":"function","doc":"Escapes the HTML entities in the given term, returning safe iodata.\n\n    iex> html_escape(\" \")\n    {:safe, [[[] | \"&lt;\"], \"hello\" | \"&gt;\"]}\n\n    iex> html_escape(~c\" \")\n    {:safe, [\"&lt;\", 104, 101, 108, 108, 111, \"&gt;\"]}\n\n    iex> html_escape(1)\n    {:safe, \"1\"}\n\n    iex> html_escape({:safe, \" \"})\n    {:safe, \" \"}","title":"Phoenix.HTML.html_escape/1","ref":"Phoenix.HTML.html#html_escape/1"},{"type":"function","doc":"Escapes HTML content to be inserted into a JavaScript string.\n\nThis function is useful in JavaScript responses when there is a need\nto escape HTML rendered from other templates, like in the following:\n\n    $(\"#container\").append(\"<%= javascript_escape(render(\"post.html\", post: @post)) %>\");\n\nIt escapes quotes (double and single), double backslashes and others.","title":"Phoenix.HTML.javascript_escape/1","ref":"Phoenix.HTML.html#javascript_escape/1"},{"type":"function","doc":"Marks the given content as raw.\n\nThis means any HTML code inside the given\nstring won't be escaped.\n\n    iex> raw(\" \")\n    {:safe, \" \"}\n    iex> raw({:safe, \" \"})\n    {:safe, \" \"}\n    iex> raw(nil)\n    {:safe, \"\"}","title":"Phoenix.HTML.raw/1","ref":"Phoenix.HTML.html#raw/1"},{"type":"type","doc":"Guaranteed to be safe","title":"Phoenix.HTML.safe/0","ref":"Phoenix.HTML.html#t:safe/0"},{"type":"function","doc":"Converts a safe result into a string.\n\nFails if the result is not safe. In such cases, you can\ninvoke `html_escape/1` or `raw/1` accordingly before.\n\nYou can combine `html_escape/1` and `safe_to_string/1`\nto convert a data structure to a escaped string:\n\n    data |> html_escape() |> safe_to_string()","title":"Phoenix.HTML.safe_to_string/1","ref":"Phoenix.HTML.html#safe_to_string/1"},{"type":"type","doc":"May be safe or unsafe (i.e. it needs to be converted)","title":"Phoenix.HTML.unsafe/0","ref":"Phoenix.HTML.html#t:unsafe/0"},{"type":"module","doc":"An EEx.Engine that guarantees templates are HTML Safe.","title":"Phoenix.HTML.Engine","ref":"Phoenix.HTML.Engine.html"},{"type":"function","doc":"Encodes the HTML templates to iodata.","title":"Phoenix.HTML.Engine.encode_to_iodata!/1","ref":"Phoenix.HTML.Engine.html#encode_to_iodata!/1"},{"type":"module","doc":"Define a `Phoenix.HTML.Form` struct and functions to interact with it.\n\nFor building actual forms in your Phoenix application, see\n[the `Phoenix.Component.form/1` component](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#form/1).","title":"Phoenix.HTML.Form","ref":"Phoenix.HTML.Form.html"},{"type":"module","doc":"The `Phoenix.HTML.Form` struct implements the `Access` behaviour.\nWhen you do `form[field]`, it returns a `Phoenix.HTML.FormField`\nstruct with the `id`, `name`, `value`, and `errors` prefilled.\n\nThe field name can be either an atom or a string. If it is an atom,\nit assumes the form keeps both data and errors as atoms. If it is a\nstring, it considers that data and errors are stored as strings for said\nfield. Forms backed by an `Ecto.Changeset` only support atom field names.\n\nIt is possible to \"access\" fields which do not exist in the source data\nstructure. A `Phoenix.HTML.FormField` struct will be dynamically created\nwith some attributes such as `name` and `id` populated.","title":"Access behaviour - Phoenix.HTML.Form","ref":"Phoenix.HTML.Form.html#module-access-behaviour"},{"type":"module","doc":"There is a protocol named `Phoenix.HTML.FormData` which can be implemented\nby any data structure that wants to be cast to the `Phoenix.HTML.Form` struct.","title":"Custom implementations - Phoenix.HTML.Form","ref":"Phoenix.HTML.Form.html#module-custom-implementations"},{"type":"function","doc":"Defines the Phoenix.HTML.Form struct.\n\nIts fields are:\n\n  * `:source` - the data structure that implements the form data protocol\n\n  * `:action` - The action that was taken against the form. This value can be\n    used to distinguish between different operations such as the user typing\n    into a form for validation, or submitting a form for a database insert.\n\n  * `:impl` - the module with the form data protocol implementation.\n    This is used to avoid multiple protocol dispatches.\n\n  * `:id` - the id to be used when generating input fields\n\n  * `:index` - the index of the struct in the form\n\n  * `:name` - the name to be used when generating input fields\n\n  * `:data` - the field used to store lookup data\n\n  * `:params` - the parameters associated with this form\n\n  * `:hidden` - a keyword list of fields that are required to\n    submit the form behind the scenes as hidden inputs\n\n  * `:options` - a copy of the options given when creating the\n    form without any form data specific key\n\n  * `:errors` - a keyword list of errors that are associated with\n    the form","title":"Phoenix.HTML.Form.__struct__/0","ref":"Phoenix.HTML.Form.html#__struct__/0"},{"type":"type","doc":"","title":"Phoenix.HTML.Form.field/0","ref":"Phoenix.HTML.Form.html#t:field/0"},{"type":"function","doc":"Receives two forms structs and checks if the given field changed.\n\nThe field will have changed if either its associated value, errors,\naction, or implementation changed. This is mostly used for optimization\nengines as an extension of the `Access` behaviour.","title":"Phoenix.HTML.Form.input_changed?/3","ref":"Phoenix.HTML.Form.html#input_changed?/3"},{"type":"function","doc":"Returns an id of a corresponding form field.\n\nThe form should either be a `Phoenix.HTML.Form` or an atom.","title":"Phoenix.HTML.Form.input_id/2","ref":"Phoenix.HTML.Form.html#input_id/2"},{"type":"function","doc":"Returns an id of a corresponding form field and value attached to it.\n\nUseful for radio buttons and inputs like multiselect checkboxes.","title":"Phoenix.HTML.Form.input_id/3","ref":"Phoenix.HTML.Form.html#input_id/3"},{"type":"function","doc":"Returns a name of a corresponding form field.\n\nThe first argument should either be a `Phoenix.HTML.Form` or an atom.","title":"Phoenix.HTML.Form.input_name/2","ref":"Phoenix.HTML.Form.html#input_name/2"},{"type":"function","doc":"iex> Phoenix.HTML.Form.input_name(:user, :first_name)\n    \"user[first_name]\"","title":"Examples - Phoenix.HTML.Form.input_name/2","ref":"Phoenix.HTML.Form.html#input_name/2-examples"},{"type":"function","doc":"Returns the HTML validations that would apply to\nthe given field.","title":"Phoenix.HTML.Form.input_validations/2","ref":"Phoenix.HTML.Form.html#input_validations/2"},{"type":"function","doc":"Returns a value of a corresponding form field.\n\nThe `form` should either be a `Phoenix.HTML.Form` or an atom.\nThe field is either a string or an atom. If the field is given\nas an atom, it will attempt to look data with atom keys. If\na string, it will look data with string keys.\n\nWhen a form is given, it will look for changes, then\nfallback to parameters, and finally fallback to the default\nstruct/map value.\n\nSince the function looks up parameter values too, there is\nno guarantee that the value will have a certain type. For\nexample, a boolean field will be sent as \"false\" as a\nparameter, and this function will return it as is. If you\nneed to normalize the result of `input_value`, see\n`normalize_value/2`.","title":"Phoenix.HTML.Form.input_value/2","ref":"Phoenix.HTML.Form.html#input_value/2"},{"type":"function","doc":"Normalizes an input `value` according to its input `type`.\n\nCertain HTML input values must be cast, or they will have idiosyncracies\nwhen they are rendered. The goal of this function is to encapsulate\nthis logic. In particular:\n\n  * For \"datetime-local\" types, it converts `DateTime` and\n    `NaiveDateTime` to strings without the second precision\n\n  * For \"checkbox\" types, it returns a boolean depending on\n    whether the input is \"true\" or not\n\n  * For \"textarea\", it prefixes a newline to ensure newlines\n    won't be ignored on submission. This requires however\n    that the textarea is rendered with no spaces after its\n    content","title":"Phoenix.HTML.Form.normalize_value/2","ref":"Phoenix.HTML.Form.html#normalize_value/2"},{"type":"function","doc":"Returns options to be used inside a select element.\n\n`options` is expected to be an enumerable which will be used to\ngenerate each `option` element. The function supports different data\nfor the individual elements:\n\n  * keyword lists - each keyword list is expected to have the keys\n    `:key` and `:value`. Additional keys such as `:disabled` may\n    be given to customize the option.\n  * two-item tuples - where the first element is an atom, string or\n    integer to be used as the option label and the second element is\n    an atom, string or integer to be used as the option value\n  * simple atom, string or integer - which will be used as both label and value\n    for the generated select","title":"Phoenix.HTML.Form.options_for_select/2","ref":"Phoenix.HTML.Form.html#options_for_select/2"},{"type":"function","doc":"If `options` is map or keyword list where the first element is a string,\natom or integer and the second element is a list or a map, it is assumed\nthe key will be wrapped in an ` ` and the value will be used to\ngenerate ` ` nested under the group.","title":"Option groups - Phoenix.HTML.Form.options_for_select/2","ref":"Phoenix.HTML.Form.html#options_for_select/2-option-groups"},{"type":"function","doc":"options_for_select([\"Admin\": \"admin\", \"User\": \"user\"], \"admin\")\n    #=>  Admin \n    #=>  User \n\nMultiple selected values:\n\n    options_for_select([\"Admin\": \"admin\", \"User\": \"user\", \"Moderator\": \"moderator\"],\n      [\"admin\", \"moderator\"])\n    #=>  Admin \n    #=>  User \n    #=>  Moderator \n\nGroups:\n\n    options_for_select([\"Europe\": [\"UK\", \"Sweden\", \"France\"], ...], nil)\n    #=>  \n    #=>    UK \n    #=>    Sweden \n    #=>    France \n    #=>  \n\nHorizontal separators can be added:\n\n    options_for_select([\"Admin\", \"User\", :hr, \"New\"], nil)\n    #=>  Admin \n    #=>  User \n    #=>  \n    #=>  New \n\n    options_for_select([\"Admin\": \"admin\", \"User\": \"user\", hr: nil, \"New\": \"new\"], nil)\n    #=>  Admin \n    #=>  User \n    #=>  \n    #=>  New","title":"Examples - Phoenix.HTML.Form.options_for_select/2","ref":"Phoenix.HTML.Form.html#options_for_select/2-examples"},{"type":"type","doc":"","title":"Phoenix.HTML.Form.t/0","ref":"Phoenix.HTML.Form.html#t:t/0"},{"type":"protocol","doc":"Converts a data structure into a `Phoenix.HTML.Form` struct.","title":"Phoenix.HTML.FormData","ref":"Phoenix.HTML.FormData.html"},{"type":"protocol","doc":"Phoenix provides integration of forms with Ecto changesets and data\nstructures via the [phoenix_ecto](https://hex.pm/packages/phoenix_ecto) package.\nIf a project was generated without Ecto support that dependency will need to be\nmanually added.","title":"Ecto integration - Phoenix.HTML.FormData","ref":"Phoenix.HTML.FormData.html#module-ecto-integration"},{"type":"function","doc":"Returns the HTML5 validations that would apply to\nthe given field.","title":"Phoenix.HTML.FormData.input_validations/3","ref":"Phoenix.HTML.FormData.html#input_validations/3"},{"type":"function","doc":"Returns the value for the given field.","title":"Phoenix.HTML.FormData.input_value/3","ref":"Phoenix.HTML.FormData.html#input_value/3"},{"type":"type","doc":"All the types that implement this protocol.","title":"Phoenix.HTML.FormData.t/0","ref":"Phoenix.HTML.FormData.html#t:t/0"},{"type":"function","doc":"Converts a data structure into a [`Phoenix.HTML.Form`](`t:Phoenix.HTML.Form.t/0`) struct.\n\nThe options have their meaning defined by the underlying\nimplementation but all shared options below are expected to\nbe implemented. All remaining options must be stored in the\nreturned struct.","title":"Phoenix.HTML.FormData.to_form/2","ref":"Phoenix.HTML.FormData.html#to_form/2"},{"type":"function","doc":"* `:as` - the value to be used as the form name\n\n  * `:id` - the ID of the form attribute. All form inputs will\n    be prefixed by the given ID","title":"Shared options - Phoenix.HTML.FormData.to_form/2","ref":"Phoenix.HTML.FormData.html#to_form/2-shared-options"},{"type":"function","doc":"Converts the field in the given form based on the data structure\ninto a list of [`Phoenix.HTML.Form`](`t:Phoenix.HTML.Form.t/0`) structs.\n\nThe options have their meaning defined by the underlying\nimplementation but all shared options below are expected to\nbe implemented. All remaining options must be stored in the\nreturned struct.","title":"Phoenix.HTML.FormData.to_form/4","ref":"Phoenix.HTML.FormData.html#to_form/4"},{"type":"function","doc":"* `:id` - the id to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form id\n\n  * `:as` - the name to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form name\n\n  * `:default` - the value to use if none is available\n\n  * `:prepend` - the values to prepend when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:append` - the values to append when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:action` - The user defined action being taken by the form, such\n    as `:validate`, `:save`, etc.","title":"Shared Options - Phoenix.HTML.FormData.to_form/4","ref":"Phoenix.HTML.FormData.html#to_form/4-shared-options"},{"type":"module","doc":"The struct returned by `form[field]`.\n\nIt has the following fields:\n\n  * `:errors` - a list of errors belonging to the field\n  * `:field` - the field name as an atom or a string\n  * `:form` - the parent `form` struct\n  * `:id` - the `id` to be used as form input as a string\n  * `:name` - the `name` to be used as form input as a string\n  * `:value` - the value for the input","title":"Phoenix.HTML.FormField","ref":"Phoenix.HTML.FormField.html"},{"type":"type","doc":"","title":"Phoenix.HTML.FormField.t/0","ref":"Phoenix.HTML.FormField.html#t:t/0"},{"type":"protocol","doc":"Defines the HTML safe protocol.\n\nIn order to promote HTML safety, Phoenix templates\ndo not use `Kernel.to_string/1` to convert data types to\nstrings in templates. Instead, Phoenix uses this\nprotocol which must be implemented by data structures\nand guarantee that a HTML safe representation is returned.\n\nFurthermore, this protocol relies on iodata, which provides\nbetter performance when sending or streaming data to the client.","title":"Phoenix.HTML.Safe","ref":"Phoenix.HTML.Safe.html"},{"type":"type","doc":"All the types that implement this protocol.","title":"Phoenix.HTML.Safe.t/0","ref":"Phoenix.HTML.Safe.html#t:t/0"},{"type":"function","doc":"","title":"Phoenix.HTML.Safe.to_iodata/1","ref":"Phoenix.HTML.Safe.html#to_iodata/1"},{"type":"extras","doc":"# Changelog","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"* Enhancements\n  * Add type to `Phoenix.HTML.FormField`\n  * Allow keyword lists in options to use nil as key/value","title":"4.2.1 (2025-02-21) - Changelog","ref":"changelog.html#4-2-1-2025-02-21"},{"type":"extras","doc":"* Enhancements\n  * Add `Phoenix.HTML.css_escape/1` to escape strings for use inside CSS selectors\n  * Add the ability to pass `:hr` to `options_for_select/2` to render a horizontal rule\n\n* Bug fixes\n  * Pass form action through in FormData implementation","title":"4.2.0 (2024-12-28) - Changelog","ref":"changelog.html#4-2-0-2024-12-28"},{"type":"extras","doc":"* Fix dependency resolution error","title":"v4.1.1 (2024-03-01) - Changelog","ref":"changelog.html#v4-1-1-2024-03-01"},{"type":"extras","doc":"* Enhancements\n  * Introduce form `:action` and consider input as changed if action changes to support better change tracking","title":"v4.1.0 (2024-02-29) - Changelog","ref":"changelog.html#v4-1-0-2024-02-29"},{"type":"extras","doc":"This version removes deprecated functionality and moved all HTML helpers to a separate library. HTML Helpers are no longer used in new apps from Phoenix v1.7, instead it relies on function components from `Phoenix.LiveView`. Older applications who wish to maintain compatibility, add `{:phoenix_html_helpers, \"~> 1.0\"}` to your `mix.exs` and then replace `use Phoenix.HTML` in your applications by:\n\n```elixir\nimport Phoenix.HTML\nimport Phoenix.HTML.Form\nuse PhoenixHTMLHelpers\n```","title":"v4.0.0 (2023-12-19) - Changelog","ref":"changelog.html#v4-0-0-2023-12-19"},{"type":"extras","doc":"* Enhancements\n  * Allow string fields on `input_changed?`","title":"v3.3.3 (2023-10-09) - Changelog","ref":"changelog.html#v3-3-3-2023-10-09"},{"type":"extras","doc":"* Enhancements\n  * Address deprecations in Elixir v1.16+\n\n* Deprecations\n  * Deprecate `inputs_for/2` and `inputs_for/3` (without anonymous functions)","title":"v3.3.2 (2023-08-10) - Changelog","ref":"changelog.html#v3-3-2-2023-08-10"},{"type":"extras","doc":"* Bug fix\n  * Set display to none on generated forms\n  * Warn for maps with atom keys","title":"v3.3.1 (2023-02-27) - Changelog","ref":"changelog.html#v3-3-1-2023-02-27"},{"type":"extras","doc":"* Enhancements\n  * Support deeply nested class lists\n  * Implement Phoenix.HTML.Safe for URI\n  * Implement Phoenix.HTML.FormData for Map\n\n* Bug fix\n  * Generate unique IDs for checkboxes based on the value\n  * Use artificial button click instead of `form.submit` in JavaScript to trigger all relevant events\n  * Fix a bug where nil/false/true attributes in `aria`/`data`/`phx` would emit empty or literal values, such as `\"true\"` and `\"false\"`. This release aligns them with all other attributes so both `nil` and `false` emit nothing. `true` emits the attribute with no value.\n\n* Deprecations\n  * `Phoenix.HTML.Tag.attributes_escape/1` is deprecated in favor of `Phoenix.HTML.attributes_escape/1`","title":"v3.3.0 (2023-02-10) - Changelog","ref":"changelog.html#v3-3-0-2023-02-10"},{"type":"extras","doc":"* Enhancements\n  * Raise if the `id` attribute is set to a number. This is actually an invalid value according to the HTML spec and it can lead to problematic client behaviour, especially in LiveView and other client frameworks.\n  * Allow `phx` attributes to be nested, similar to `aria` and `data` attributes\n  * Allow hidden fields in forms to be a list of values","title":"v3.2.0 (2021-12-18) - Changelog","ref":"changelog.html#v3-2-0-2021-12-18"},{"type":"extras","doc":"* Bug fix\n  * Do not submit data-method links if default has been prevented\n* Deprecations\n  * Deprecate `~E` and `Phoenix.HTML.Tag.attributes_escape/1`\n  * Remove deprecated `Phoenix.HTML.Link.link/1`","title":"v3.1.0 (2021-10-23) - Changelog","ref":"changelog.html#v3-1-0-2021-10-23"},{"type":"extras","doc":"* Bug fix\n  * Ensure `class={@class}` in HEEx templates and `:class` attribute in `content_tag` are properly escaped against XSS","title":"v3.0.4 (2021-09-23) - Changelog","ref":"changelog.html#v3-0-4-2021-09-23"},{"type":"extras","doc":"* Bug fix\n  * Fix sorting of attributes in `tag`/`content_tag`","title":"v3.0.3 (2021-09-04) - Changelog","ref":"changelog.html#v3-0-3-2021-09-04"},{"type":"extras","doc":"* Enhancements\n  * Support maps on `Phoenix.HTML.Tag.attributes_escape/1`","title":"v3.0.2 (2021-08-19) - Changelog","ref":"changelog.html#v3-0-2-2021-08-19"},{"type":"extras","doc":"* Enhancements\n  * Add `Phoenix.HTML.Tag.csrf_input_tag/2`","title":"v3.0.1 (2021-08-14) - Changelog","ref":"changelog.html#v3-0-1-2021-08-14"},{"type":"extras","doc":"* Enhancements\n  * Allow extra html attributes on the `:prompt` option in `select`\n  * Make `Plug` an optional dependency\n  * Prefix form id on inputs when it is given to `form_for/3`\n  * Allow `%URI{}` to be passed to `link/2` and `button/2` as `:to`\n  * Expose `Phoenix.HTML.Tag.csrf_token_value/1`\n  * Add `Phoenix.HTML.Tag.attributes_escape/1`\n\n* Bug fixes\n  * Honor the `form` attribute when creating hidden checkbox input\n  * Use `to_iso8601` as the standard implementation for safe dates and times\n\n* Deprecations\n  * `form_for` without an anonymous function has been deprecated. v3.0 has deprecated the usage, v3.1 will emit warnings, and v3.2 will fully remove the functionality\n\n* Backwards incompatible changes\n  * Strings given as attributes keys in `tag` and `content_tag` are now emitted as is (without being dasherized) and are also HTML escaped\n  * Prefix form id on inputs when it is given to `form_for/3`\n  * By default dates and times will format to the `to_iso8601` functions provided by their implementation\n  * Do not include `csrf-param` and `method-param` in generated `csrf_meta_tag`\n  * Remove deprecated `escape_javascript` in favor of `javascript_escape`\n  * Remove deprecated `field_value` in favor of `input_value`\n  * Remove deprecated `field_name` in favor of `input_name`\n  * Remove deprecated `field_id` in favor of `input_id`","title":"v3.0.0 (2021-08-06) - Changelog","ref":"changelog.html#v3-0-0-2021-08-06"},{"type":"extras","doc":"* Bug fixes\n  * Fix warnings on Elixir v1.12","title":"v2.14.3 (2020-12-12) - Changelog","ref":"changelog.html#v2-14-3-2020-12-12"},{"type":"extras","doc":"* Deprecations\n  * Deprecate `Phoenix`-specific assigns `:view_module` and `:view_template`","title":"v2.14.2 (2020-04-30) - Changelog","ref":"changelog.html#v2-14-2-2020-04-30"},{"type":"extras","doc":"* Enhancements\n  * Add `Phoenix.HTML.Form.options_for_select/2`\n  * Add `Phoenix.HTML.Form.inputs_for/3`\n\n* Bug fixes\n  * Disable hidden input for disabled checkboxes","title":"v2.14.1 (2020-03-20) - Changelog","ref":"changelog.html#v2-14-1-2020-03-20"},{"type":"extras","doc":"* Enhancements\n  * Remove enforce_utf8 workaround on forms as it is no longer required by browser\n  * Remove support tuple-based date/time with microseconds calendar types\n  * Allow strings as first element in `content_tag`\n  * Add `:srcset` support to `img_tag`\n  * Allow `inputs_for` to skip hidden fields","title":"v2.14.0 (2020-01-28) - Changelog","ref":"changelog.html#v2-14-0-2020-01-28"},{"type":"extras","doc":"* Bug fixes\n  * Fix invalid :line in Elixir v1.10.0","title":"v2.13.4 (2020-01-28) - Changelog","ref":"changelog.html#v2-13-4-2020-01-28"},{"type":"extras","doc":"* Enhancements\n  * Add atom support to FormData\n\n* Bug fixes\n  * Keep proper line numbers on .eex templates for proper coverage","title":"v2.13.3 (2019-05-31) - Changelog","ref":"changelog.html#v2-13-3-2019-05-31"},{"type":"extras","doc":"* Bug fixes\n  * Stop event propagation when confirm dialog is canceled","title":"v2.13.2 (2019-03-29) - Changelog","ref":"changelog.html#v2-13-2-2019-03-29"},{"type":"extras","doc":"* Enhancements\n  * Allow safe content to be given to label\n  * Also escale template literals in `javascript_escape/1`\n\n* Bug fixes\n  * Fix deprecation warnings to point to the correct alternative","title":"v2.13.1 (2019-01-05) - Changelog","ref":"changelog.html#v2-13-1-2019-01-05"},{"type":"extras","doc":"* Enhancements\n  * Require Elixir v1.5+ for more efficient template compilation/rendering\n  * Add `Phoenix.HTML.Engine.encode_to_iodata!/1`\n  * Add `Phoenix.HTML.Form.form_for/3` that works without an anonymous function\n\n* Deprecations\n  * Deprecate `Phoenix.HTML.escape_javascript/1` in favor of `Phoenix.HTML.javascript_escape/1` for consistency","title":"v2.13.0 (2018-12-09) - Changelog","ref":"changelog.html#v2-13-0-2018-12-09"},{"type":"extras","doc":"* Enhancements\n  * Configurable and extendable data-confirm behaviour\n  * Allow data-confirm with submit buttons\n  * Support ISO 8601 formatted strings for date and time values\n\n* Bug fixes\n  * Provide a default id of the field name for `@conn` based forms","title":"v2.12.0 (2018-08-06) - Changelog","ref":"changelog.html#v2-12-0-2018-08-06"},{"type":"extras","doc":"* Enhancements\n  * Support custom precision on time input\n\n* Bug fixes\n  * Do not raise when `:` is part of a path on link/button attributes","title":"v2.11.2 (2018-04-13) - Changelog","ref":"changelog.html#v2-11-2-2018-04-13"},{"type":"extras","doc":"* Enhancements\n  * Add `label/1`\n  * Copy the target attribute of the link in the generated JS form\n\n* Bug fixes\n  * Support any value that is html escapable in `radio_button`","title":"v2.11.1 (2018-03-20) - Changelog","ref":"changelog.html#v2-11-1-2018-03-20"},{"type":"extras","doc":"* Enhancements\n  * Add date, datetime-local and time input types\n  * Enable string keys to be usable with forms\n  * Support carriage return in `text_to_html`\n  * Add support for HTML5 boolean attributes to `content_tag` and `tag`\n  * Improve performance by relying on `html_safe_to_iodata/1`\n  * Protect against CSRF tokens leaking across hosts when the POST URL is dynamic\n  * Require `to` attribute in links and buttons to explicitly pass protocols as a separate option for safety reasons\n\n* Bug fixes\n  * Guarantee `input_name/2` always returns strings\n  * Improve handling of uncommon whitespace and null in `escape_javascript`\n  * Escape value attribute so it is never treated as a boolean\n\n* Backwards incompatible changes\n  * The :csrf_token_generator configuration in the Phoenix.HTML app no longer works due to the improved security mechanisms","title":"v2.11.0 (2018-03-09) - Changelog","ref":"changelog.html#v2-11-0-2018-03-09"},{"type":"extras","doc":"* Enhancements\n  * Do not require the :as option in form_for","title":"v2.10.5 (2017-11-08) - Changelog","ref":"changelog.html#v2-10-5-2017-11-08"},{"type":"extras","doc":"* Bug fixes\n  * Fix formatting of days in datetime_builder","title":"v2.10.4 (2017-08-15) - Changelog","ref":"changelog.html#v2-10-4-2017-08-15"},{"type":"extras","doc":"* Enhancements\n  * Allow specifying a custom CSRF token generator\n\n* Bug fixes\n  * Do not submit `method: :get` in buttons as \"post\"","title":"v2.10.3 (2017-07-30) - Changelog","ref":"changelog.html#v2-10-3-2017-07-30"},{"type":"extras","doc":"* Bug fixes\n  * Traverse DOM elements up when handling data-method","title":"v2.10.2 (2017-07-24) - Changelog","ref":"changelog.html#v2-10-2-2017-07-24"},{"type":"extras","doc":"* Bug fixes\n  * Only generate CSRF token if necessary","title":"v2.10.1 (2017-07-22) - Changelog","ref":"changelog.html#v2-10-1-2017-07-22"},{"type":"extras","doc":"* Enhancements\n  * Support custom attributes in options in select\n\n* Bug fixes\n  * Accept non-binary values in textarea's content\n  * Allow nested forms on the javascript side. This means `link` and `button` no longer generate a child form such as the `:form` option has no effect and \"data-submit=parent\" is no longer supported. Instead \"data-to\" and \"data-method\" are set on the entities and the form is generated on the javascript side of things","title":"v2.10.0 (2017-07-21) - Changelog","ref":"changelog.html#v2-10-0-2017-07-21"},{"type":"extras","doc":"* Bug fixes\n  * Once again support any name for atom forms","title":"v2.9.3 (2016-12-24) - Changelog","ref":"changelog.html#v2-9-3-2016-12-24"},{"type":"extras","doc":"* Bug fixes\n  * Always read from `form.params` and then from `:selected` in `select` and `multiple_select` before falling back to `input_value/2`","title":"v2.9.2 (2016-12-24) - Changelog","ref":"changelog.html#v2-9-2-2016-12-24"},{"type":"extras","doc":"* Bug fixes\n  * Implement proper `input_value/3` callback","title":"v2.9.1 (2016-12-20) - Changelog","ref":"changelog.html#v2-9-1-2016-12-20"},{"type":"extras","doc":"* Enhancements\n  * Add `img_tag/2` helper to `Phoenix.HTML.Tag`\n  * Submit nearest form even if not direct descendent\n  * Use more iodata for `tag/2` and `content_tag/3`\n  * Add `input_value/3`, `input_id/2` and `input_name/2` as a unified API around the input (alongside `input_type/3` and `input_validations/2`)","title":"v2.9.0 (2016-12-19) - Changelog","ref":"changelog.html#v2-9-0-2016-12-19"},{"type":"extras","doc":"* Enhancements\n  * Add `csrf_meta_tag/0` helper to `Phoenix.HTML.Tag`\n  * Allow passing a `do:` option to `Phoenix.HTML.Link.button/2`","title":"v2.8.0 (2016-11-15) - Changelog","ref":"changelog.html#v2-8-0-2016-11-15"},{"type":"extras","doc":"* Enhancements\n  * Render button tags for form submits and in the `button/2` function\n  * Allow `submit/2` and `button/2` to receive `do` blocks\n  * Support the `:multiple` option in `file_input/3`\n  * Remove previously deprecated and unused `model` field","title":"v2.7.0 (2016-09-21) - Changelog","ref":"changelog.html#v2-7-0-2016-09-21"},{"type":"extras","doc":"* Enhancements\n  * Remove warnings on v1.4\n\n* Bug fixes\n  * Ensure some contents are properly escaped as an integer\n  * Ensure JavaScript data-submit events bubble up until it finds the proper parent","title":"v2.6.1 (2016-07-08) - Changelog","ref":"changelog.html#v2-6-1-2016-07-08"},{"type":"extras","doc":"* Enhancements\n  * Raise helpful error when using invalid iodata\n  * Inline date/time API with Elixir v1.3 Calendar types\n  * Add `:insert_brs` option to `text_to_html/2`\n  * Run on Erlang 19 without warnings\n\n* Client-side changes\n  * Use event delegation in `phoenix_html.js`\n  * Drop IE8 support on `phoenix_html.js`\n\n* Backwards incompatible changes\n  * `:min`, `:sec` option in `Phoenix.HTML.Form` (`datetime_select/3` and `time_select/3`) are no longer supported. Use `:minute` or `:second` instead.","title":"v2.6.0 (2016-06-16) - Changelog","ref":"changelog.html#v2-6-0-2016-06-16"},{"type":"extras","doc":"* Bug fixes\n  * Ensure multipart files work with inputs_for","title":"v2.5.1 (2016-03-12) - Changelog","ref":"changelog.html#v2-5-1-2016-03-12"},{"type":"extras","doc":"* Enhancements\n  * Introduce `form.data` field instead of `form.model`. Currently those values are kept in sync then the form is built but `form.model` will be deprecated in the long term","title":"v2.5.0 (2016-01-28) - Changelog","ref":"changelog.html#v2-5-0-2016-01-28"},{"type":"extras","doc":"* Enhancements\n  * Add `rel=nofollow` auto generation for non-get links\n  * Introduce `:selected` option for `select`  and `multiple_select`\n\n* Bug fixes\n  * Fix safe engine incorrectly marking safe code as unsafe when last expression is `<% ... %>`","title":"v2.4.0 (2016-01-21) - Changelog","ref":"changelog.html#v2-4-0-2016-01-21"},{"type":"extras","doc":"* Enhancements\n  * Add `escape_javascript/1`\n  * Add helpful error message when using unknown `@inner` assign\n  * Add `Phoenix.HTML.Format.text_to_html/2`","title":"v2.3.0 (2015-12-16) - Changelog","ref":"changelog.html#v2-3-0-2015-12-16"},{"type":"extras","doc":"* Bug fix\n  * Allow the `:name` to be given in forms. For this, using `:name` to configure the underlying input name prefix has been deprecated in favor of `:as`","title":"v2.2.0 (2015-09-01) - Changelog","ref":"changelog.html#v2-2-0-2015-09-01"},{"type":"extras","doc":"* Bug fix\n  * Do not include values in `password_input/3`","title":"v2.1.2 (2015-08-22) - Changelog","ref":"changelog.html#v2-1-2-2015-08-22"},{"type":"extras","doc":"* Enhancements\n  * Allow nil in `raw/1`\n  * Allow block options in `label/3`\n  * Introduce `:skip_deleted` in `inputs_for/4`","title":"v2.1.1 (2015-08-15) - Changelog","ref":"changelog.html#v2-1-1-2015-08-15"},{"type":"extras","doc":"* Enhancements\n  * Add an index field to forms to be used by `inputs_for/4` collections","title":"v2.1.0 (2015-08-06) - Changelog","ref":"changelog.html#v2-1-0-2015-08-06"},{"type":"extras","doc":"* Bug fix\n  * Include web directory in Hex package","title":"v2.0.1 (2015-07-31) - Changelog","ref":"changelog.html#v2-0-1-2015-07-31"},{"type":"extras","doc":"* Enhancements\n  * No longer generate onclick attributes.\n\n    The main motivation for this is to provide support\n    for Content Security Policy, which recommends\n    disabling all inline scripts in a page.\n\n    We took the opportunity to also add support for\n    data-confirm in `link/2`.","title":"v2.0.0 (2015-07-30) - Changelog","ref":"changelog.html#v2-0-0-2015-07-30"},{"type":"extras","doc":"* Enhancements\n  * Support `input_type/2` and `input_validations/2` as reflection mechanisms","title":"v1.4.0 (2015-07-26) - Changelog","ref":"changelog.html#v1-4-0-2015-07-26"},{"type":"extras","doc":"* Enhancements\n  * Add `Phoenix.HTML.Form.inputs_for/4` support\n  * Add multiple select support\n  * Add reset input\n  * Infer default text context for labels","title":"v1.3.0 (2015-07-23) - Changelog","ref":"changelog.html#v1-3-0-2015-07-23"},{"type":"extras","doc":"* Bug fix\n  * Ensure nil parameters are not discarded when rendering input","title":"v1.2.1 (2015-06-02) - Changelog","ref":"changelog.html#v1-2-1-2015-06-02"},{"type":"extras","doc":"* Enhancements\n  * Add `label/3` for generating a label tag within a form","title":"v1.2.0 (2015-05-30) - Changelog","ref":"changelog.html#v1-2-0-2015-05-30"},{"type":"extras","doc":"* Enhancements\n  * Allow do/end syntax with `link/2`\n  * Raise on missing assigns","title":"v1.1.0 (2015-05-20) - Changelog","ref":"changelog.html#v1-1-0-2015-05-20"},{"type":"extras","doc":"* Bug fixes\n  * Avoid variable clash in Phoenix.HTML engine buffers","title":"v1.0.1 - Changelog","ref":"changelog.html#v1-0-1"},{"type":"extras","doc":"* Enhancements\n  * Provides an EEx engine with HTML safe rendering\n  * Provides a `Phoenix.HTML.Safe` protocol\n  * Provides a `Phoenix.HTML.FormData` protocol\n  * Provides functions for generating tags, links and form builders in a safe way","title":"v1.0.0 - Changelog","ref":"changelog.html#v1-0-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}